// src/utils/can-encode.ts
async function canEncode(options) {
  try {
    if (!isWebCodecsSupported()) {
      return false;
    }
    if (!options) {
      return await testDefaultConfiguration();
    }
    if (options.video?.codec) {
      const videoSupported = await testVideoCodecSupport(options.video.codec, options);
      if (!videoSupported) {
        return false;
      }
    } else {
      const videoSupported = await testVideoCodecSupport("avc", options);
      if (!videoSupported) {
        return false;
      }
    }
    if (options.audio !== false) {
      const audioCodec = typeof options.audio === "object" && options.audio?.codec ? options.audio.codec : "aac";
      const audioSupported = await testAudioCodecSupport(audioCodec, options);
      if (!audioSupported) {
        return false;
      }
    }
    return true;
  } catch (error) {
    console.warn("canEncode error:", error);
    return false;
  }
}
function isWebCodecsSupported() {
  try {
    return typeof VideoEncoder !== "undefined" && typeof AudioEncoder !== "undefined" && typeof VideoFrame !== "undefined" && typeof AudioData !== "undefined";
  } catch {
    return false;
  }
}
async function testDefaultConfiguration() {
  try {
    const videoConfig = {
      codec: "avc1.42001e",
      // Baseline Profile Level 3.0
      width: 640,
      height: 480,
      bitrate: 1e6,
      framerate: 30
    };
    const videoSupport = await VideoEncoder.isConfigSupported(videoConfig);
    if (!videoSupport.supported) {
      return false;
    }
    const audioConfig = {
      codec: "mp4a.40.2",
      // AAC-LC
      sampleRate: 48e3,
      numberOfChannels: 2,
      bitrate: 128e3
    };
    const audioSupport = await AudioEncoder.isConfigSupported(audioConfig);
    return audioSupport.supported || false;
  } catch {
    return false;
  }
}
async function testVideoCodecSupport(codec, options) {
  try {
    const codecString = getVideoCodecString(codec);
    const config = {
      codec: codecString,
      width: options?.width || 640,
      height: options?.height || 480,
      bitrate: options?.video?.bitrate || 1e6,
      framerate: options?.frameRate || 30
    };
    if (options?.video?.hardwareAcceleration) {
      config.hardwareAcceleration = options.video.hardwareAcceleration;
    }
    if (options?.video?.latencyMode) {
      config.latencyMode = options.video.latencyMode;
    }
    const support = await VideoEncoder.isConfigSupported(config);
    return support.supported || false;
  } catch {
    return false;
  }
}
async function testAudioCodecSupport(codec, options) {
  try {
    const codecString = getAudioCodecString(codec);
    const audioOptions = typeof options?.audio === "object" ? options.audio : {};
    const config = {
      codec: codecString,
      sampleRate: audioOptions.sampleRate || 48e3,
      numberOfChannels: audioOptions.channels || 2,
      bitrate: audioOptions.bitrate || 128e3
    };
    if (codec === "aac" && audioOptions.bitrateMode) {
      config.bitrateMode = audioOptions.bitrateMode;
    }
    const support = await AudioEncoder.isConfigSupported(config);
    return support.supported || false;
  } catch {
    return false;
  }
}
function getVideoCodecString(codec) {
  switch (codec) {
    case "avc":
      return "avc1.42001e";
    // H.264 Baseline Profile Level 3.0
    case "hevc":
      return "hev1.1.6.L93.B0";
    // H.265 Main Profile
    case "vp9":
      return "vp09.00.10.08";
    // VP9 Profile 0
    case "vp8":
      return "vp8";
    // VP8
    case "av1":
      return "av01.0.04M.08";
    // AV1 Main Profile Level 4.0
    default:
      return codec;
  }
}
function getAudioCodecString(codec) {
  switch (codec) {
    case "aac":
      return "mp4a.40.2";
    // AAC-LC
    case "opus":
      return "opus";
    // Opus
    default:
      return codec;
  }
}
async function canEncodeWithProfile(videoCodec, audioCodec, profile) {
  const result = { video: false, audio: false, overall: false };
  try {
    if (videoCodec) {
      const videoConfig = {
        codec: getVideoCodecString(videoCodec),
        width: profile?.width || 1920,
        height: profile?.height || 1080,
        bitrate: profile?.videoBitrate || 2e6,
        framerate: profile?.framerate || 30
      };
      const videoSupport = await VideoEncoder.isConfigSupported(videoConfig);
      result.video = videoSupport.supported || false;
    }
    if (audioCodec) {
      const audioConfig = {
        codec: getAudioCodecString(audioCodec),
        sampleRate: 48e3,
        numberOfChannels: 2,
        bitrate: profile?.audioBitrate || 128e3
      };
      const audioSupport = await AudioEncoder.isConfigSupported(audioConfig);
      result.audio = audioSupport.supported || false;
    } else {
      result.audio = true;
    }
    result.overall = result.video && result.audio;
    return result;
  } catch (error) {
    console.warn("canEncodeWithProfile error:", error);
    return result;
  }
}
export {
  canEncode,
  canEncodeWithProfile
};
//# sourceMappingURL=can-encode.js.map