// src/utils/config-parser.ts
async function inferAndBuildConfig(source, options) {
  const inferredConfig = await inferConfigFromSource(source);
  const mergedOptions = mergeWithUserOptions(inferredConfig, options);
  const configWithPreset = applyQualityPreset(mergedOptions, options?.quality);
  return convertToEncoderConfig(configWithPreset);
}
async function inferConfigFromSource(source) {
  const config = {
    frameRate: 30,
    // デフォルト値
    container: "mp4"
    // デフォルト値
  };
  try {
    const firstFrame = await getFirstFrame(source);
    if (firstFrame) {
      const dimensions = getFrameDimensions(firstFrame);
      config.width = dimensions.width;
      config.height = dimensions.height;
    }
    if (source instanceof MediaStream) {
      const audioTracks = source.getAudioTracks();
      if (audioTracks.length === 0) {
        config.audio = false;
      } else {
        const audioTrack = audioTracks[0];
        const settings = audioTrack.getSettings();
        config.audio = {
          sampleRate: settings.sampleRate || 48e3,
          channels: settings.channelCount || 2
        };
      }
    }
  } catch (error) {
    config.width = 640;
    config.height = 480;
  }
  return config;
}
function mergeWithUserOptions(inferredConfig, userOptions) {
  return {
    // 推定された設定をベースに
    ...inferredConfig,
    // ユーザー指定の設定で上書き
    ...userOptions,
    // ネストしたオブジェクトは個別にマージ
    video: {
      ...inferredConfig.video,
      ...userOptions?.video
    },
    audio: userOptions?.audio === false ? false : {
      ...inferredConfig.audio,
      ...userOptions?.audio
    }
  };
}
function applyQualityPreset(config, quality) {
  if (!quality) return config;
  const width = config.width || 640;
  const height = config.height || 480;
  const pixels = width * height;
  const basePixelsPerSecond = pixels * (config.frameRate || 30);
  let videoBitrate;
  let audioBitrate;
  switch (quality) {
    case "low":
      videoBitrate = Math.max(5e5, basePixelsPerSecond * 0.1);
      audioBitrate = 64e3;
      break;
    case "medium":
      videoBitrate = Math.max(1e6, basePixelsPerSecond * 0.2);
      audioBitrate = 128e3;
      break;
    case "high":
      videoBitrate = Math.max(2e6, basePixelsPerSecond * 0.4);
      audioBitrate = 192e3;
      break;
    case "lossless":
      videoBitrate = Math.max(1e7, basePixelsPerSecond * 1);
      audioBitrate = 32e4;
      break;
    default:
      return config;
  }
  return {
    ...config,
    video: {
      ...config.video,
      bitrate: config.video?.bitrate || videoBitrate
    },
    audio: config.audio === false ? false : {
      ...config.audio,
      bitrate: config.audio?.bitrate || audioBitrate
    }
  };
}
function convertToEncoderConfig(options) {
  const config = {
    width: options.width || 640,
    height: options.height || 480,
    frameRate: options.frameRate || 30,
    videoBitrate: options.video?.bitrate || 1e6,
    audioBitrate: options.audio === false ? 0 : options.audio?.bitrate || 128e3,
    sampleRate: options.audio === false ? 0 : options.audio?.sampleRate || 48e3,
    channels: options.audio === false ? 0 : options.audio?.channels || 2,
    container: options.container || "mp4",
    codec: {
      video: options.video?.codec || "avc",
      audio: options.audio === false ? void 0 : options.audio?.codec || "aac"
    },
    latencyMode: options.video?.latencyMode || "quality",
    hardwareAcceleration: options.video?.hardwareAcceleration || "no-preference",
    keyFrameInterval: options.video?.keyFrameInterval,
    audioBitrateMode: options.audio === false ? void 0 : options.audio?.bitrateMode || "variable"
  };
  return config;
}
async function getFirstFrame(source) {
  if (Array.isArray(source)) {
    return source.length > 0 ? source[0] : null;
  }
  if (source instanceof MediaStream) {
    const videoTracks = source.getVideoTracks();
    if (videoTracks.length > 0) {
      const settings = videoTracks[0].getSettings();
      if (settings.width && settings.height) {
        return {
          displayWidth: settings.width,
          displayHeight: settings.height
        };
      }
    }
    return null;
  }
  if (Symbol.asyncIterator in source) {
    for await (const frame of source) {
      return frame;
    }
    return null;
  }
  return null;
}
function getFrameDimensions(frame) {
  if (!frame) {
    return { width: 640, height: 480 };
  }
  if (frame instanceof VideoFrame) {
    return {
      width: frame.displayWidth || frame.codedWidth,
      height: frame.displayHeight || frame.codedHeight
    };
  }
  if (frame instanceof HTMLCanvasElement || frame instanceof OffscreenCanvas) {
    return { width: frame.width, height: frame.height };
  }
  if (frame instanceof ImageBitmap) {
    return { width: frame.width, height: frame.height };
  }
  if (frame instanceof ImageData) {
    return { width: frame.width, height: frame.height };
  }
  if ("displayWidth" in frame && "displayHeight" in frame) {
    return {
      width: frame.displayWidth,
      height: frame.displayHeight
    };
  }
  return { width: 640, height: 480 };
}
export {
  inferAndBuildConfig
};
//# sourceMappingURL=config-parser.js.map