"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/core/encode.ts
var encode_exports = {};
__export(encode_exports, {
  encode: () => encode
});
module.exports = __toCommonJS(encode_exports);

// src/types.ts
var EncodeError = class _EncodeError extends Error {
  constructor(type, message, cause) {
    super(message);
    this.name = "EncodeError";
    this.type = type;
    this.cause = cause;
    Object.setPrototypeOf(this, _EncodeError.prototype);
  }
};

// src/utils/config-parser.ts
async function inferAndBuildConfig(source, options) {
  const inferredConfig = await inferConfigFromSource(source);
  const mergedOptions = mergeWithUserOptions(inferredConfig, options);
  const configWithPreset = applyQualityPreset(mergedOptions, options?.quality);
  return convertToEncoderConfig(configWithPreset);
}
async function inferConfigFromSource(source) {
  const config = {
    frameRate: 30,
    // デフォルト値
    container: "mp4"
    // デフォルト値
  };
  try {
    const firstFrame = await getFirstFrame(source);
    if (firstFrame) {
      const dimensions = getFrameDimensions(firstFrame);
      config.width = dimensions.width;
      config.height = dimensions.height;
    }
    if (source instanceof MediaStream) {
      const audioTracks = source.getAudioTracks();
      if (audioTracks.length === 0) {
        config.audio = false;
      } else {
        const audioTrack = audioTracks[0];
        const settings = audioTrack.getSettings();
        config.audio = {
          sampleRate: settings.sampleRate || 48e3,
          channels: settings.channelCount || 2
        };
      }
    }
  } catch (error) {
    config.width = 640;
    config.height = 480;
  }
  return config;
}
function mergeWithUserOptions(inferredConfig, userOptions) {
  return {
    // 推定された設定をベースに
    ...inferredConfig,
    // ユーザー指定の設定で上書き
    ...userOptions,
    // ネストしたオブジェクトは個別にマージ
    video: {
      ...inferredConfig.video,
      ...userOptions?.video
    },
    audio: userOptions?.audio === false ? false : {
      ...inferredConfig.audio,
      ...userOptions?.audio
    }
  };
}
function applyQualityPreset(config, quality) {
  if (!quality) return config;
  const width = config.width || 640;
  const height = config.height || 480;
  const pixels = width * height;
  const basePixelsPerSecond = pixels * (config.frameRate || 30);
  let videoBitrate;
  let audioBitrate;
  switch (quality) {
    case "low":
      videoBitrate = Math.max(5e5, basePixelsPerSecond * 0.1);
      audioBitrate = 64e3;
      break;
    case "medium":
      videoBitrate = Math.max(1e6, basePixelsPerSecond * 0.2);
      audioBitrate = 128e3;
      break;
    case "high":
      videoBitrate = Math.max(2e6, basePixelsPerSecond * 0.4);
      audioBitrate = 192e3;
      break;
    case "lossless":
      videoBitrate = Math.max(1e7, basePixelsPerSecond * 1);
      audioBitrate = 32e4;
      break;
    default:
      return config;
  }
  return {
    ...config,
    video: {
      ...config.video,
      bitrate: config.video?.bitrate || videoBitrate
    },
    audio: config.audio === false ? false : {
      ...config.audio,
      bitrate: config.audio?.bitrate || audioBitrate
    }
  };
}
function convertToEncoderConfig(options) {
  const config = {
    width: options.width || 640,
    height: options.height || 480,
    frameRate: options.frameRate || 30,
    videoBitrate: options.video?.bitrate || 1e6,
    audioBitrate: options.audio === false ? 0 : options.audio?.bitrate || 128e3,
    sampleRate: options.audio === false ? 0 : options.audio?.sampleRate || 48e3,
    channels: options.audio === false ? 0 : options.audio?.channels || 2,
    container: options.container || "mp4",
    codec: {
      video: options.video?.codec || "avc",
      audio: options.audio === false ? void 0 : options.audio?.codec || "aac"
    },
    latencyMode: options.video?.latencyMode || "quality",
    hardwareAcceleration: options.video?.hardwareAcceleration || "no-preference",
    keyFrameInterval: options.video?.keyFrameInterval,
    audioBitrateMode: options.audio === false ? void 0 : options.audio?.bitrateMode || "variable"
  };
  return config;
}
async function getFirstFrame(source) {
  if (Array.isArray(source)) {
    return source.length > 0 ? source[0] : null;
  }
  if (source instanceof MediaStream) {
    const videoTracks = source.getVideoTracks();
    if (videoTracks.length > 0) {
      const settings = videoTracks[0].getSettings();
      if (settings.width && settings.height) {
        return {
          displayWidth: settings.width,
          displayHeight: settings.height
        };
      }
    }
    return null;
  }
  if (Symbol.asyncIterator in source) {
    for await (const frame of source) {
      return frame;
    }
    return null;
  }
  return null;
}
function getFrameDimensions(frame) {
  if (!frame) {
    return { width: 640, height: 480 };
  }
  if (frame instanceof VideoFrame) {
    return {
      width: frame.displayWidth || frame.codedWidth,
      height: frame.displayHeight || frame.codedHeight
    };
  }
  if (frame instanceof HTMLCanvasElement || frame instanceof OffscreenCanvas) {
    return { width: frame.width, height: frame.height };
  }
  if (frame instanceof ImageBitmap) {
    return { width: frame.width, height: frame.height };
  }
  if (frame instanceof ImageData) {
    return { width: frame.width, height: frame.height };
  }
  if ("displayWidth" in frame && "displayHeight" in frame) {
    return {
      width: frame.displayWidth,
      height: frame.displayHeight
    };
  }
  return { width: 640, height: 480 };
}

// src/utils/worker-manager.ts
var workerInstance = null;
var workerBlobUrl = null;
function getWorker() {
  if (!workerInstance) {
    workerInstance = createInlineWorker();
  }
  return workerInstance;
}
function createInlineWorker() {
  try {
    const workerSource = getWorkerSource();
    const blob = new Blob([workerSource], { type: "application/javascript" });
    workerBlobUrl = URL.createObjectURL(blob);
    const worker = new Worker(workerBlobUrl, { type: "module" });
    worker.onerror = (event) => {
      console.error("Worker error:", event);
      throw new EncodeError("worker-error", `Worker error: ${event.message}`);
    };
    return worker;
  } catch (error) {
    throw new EncodeError(
      "initialization-failed",
      "Failed to create inline worker",
      error
    );
  }
}
function getWorkerSource() {
  return `
    // WebCodecs Encoder Worker (Inline)
    
    // Import statements for WebCodecs APIs
    // Note: In a real implementation, this would be the actual worker.ts content
    
    const mp4Muxer = null; // Placeholder - would import mp4-muxer
    const webmMuxer = null; // Placeholder - would import webm-muxer
    
    let videoEncoder = null;
    let audioEncoder = null;
    let muxer = null;
    let config = null;
    
    self.onmessage = async function(event) {
      const { type, ...data } = event.data;
      
      try {
        switch (type) {
          case 'initialize':
            config = data.config;
            await initialize(config);
            self.postMessage({ type: 'initialized' });
            break;
            
          case 'addVideoFrame':
            if (videoEncoder) {
              await encodeVideoFrame(data.frame, data.timestamp);
            }
            break;
            
          case 'addAudioData':
            if (audioEncoder) {
              await encodeAudioData(data);
            }
            break;
            
          case 'finalize':
            const result = await finalize();
            self.postMessage({ type: 'finalized', output: result });
            break;
            
          case 'cancel':
            cleanup();
            self.postMessage({ type: 'cancelled' });
            break;
            
          default:
            console.warn('Unknown message type:', type);
        }
      } catch (error) {
        self.postMessage({ 
          type: 'error', 
          errorDetail: {
            message: error.message,
            type: 'encoding-failed',
            stack: error.stack
          }
        });
      }
    };
    
    async function initialize(config) {
      // Initialize encoders and muxer based on config
      // This is a placeholder - actual implementation would be much more complex
      console.log('Worker initialized with config:', config);
    }
    
    async function encodeVideoFrame(frame, timestamp) {
      // Encode video frame
      console.log('Encoding video frame at timestamp:', timestamp);
    }
    
    async function encodeAudioData(data) {
      // Encode audio data
      console.log('Encoding audio data:', data);
    }
    
    async function finalize() {
      // Finalize encoding and return result
      console.log('Finalizing encoding');
      return new Uint8Array(0); // Placeholder
    }
    
    function cleanup() {
      // Clean up encoders and muxer
      if (videoEncoder) {
        videoEncoder.close();
        videoEncoder = null;
      }
      if (audioEncoder) {
        audioEncoder.close();
        audioEncoder = null;
      }
      muxer = null;
    }
  `;
}
var WorkerCommunicator = class {
  constructor() {
    this.messageHandlers = /* @__PURE__ */ new Map();
    this.worker = getWorker();
    this.worker.onmessage = this.handleMessage.bind(this);
  }
  handleMessage(event) {
    const { type, ...data } = event.data;
    const handler = this.messageHandlers.get(type);
    if (handler) {
      handler(data);
    }
  }
  /**
   * メッセージハンドラーを登録
   */
  on(type, handler) {
    this.messageHandlers.set(type, handler);
  }
  /**
   * メッセージハンドラーを解除
   */
  off(type) {
    this.messageHandlers.delete(type);
  }
  /**
   * ワーカーにメッセージを送信
   */
  send(type, data = {}) {
    this.worker.postMessage({ type, ...data });
  }
  /**
   * 通信を終了
   */
  terminate() {
    this.messageHandlers.clear();
  }
};

// src/core/encode.ts
async function encode(source, options) {
  let communicator = null;
  try {
    const config = await inferAndBuildConfig(source, options);
    communicator = new WorkerCommunicator();
    const result = await performEncoding(communicator, source, config, options);
    return result;
  } catch (error) {
    const encodeError = error instanceof EncodeError ? error : new EncodeError(
      "encoding-failed",
      `Encoding failed: ${error instanceof Error ? error.message : String(error)}`,
      error
    );
    if (options?.onError) {
      options.onError(encodeError);
    }
    throw encodeError;
  } finally {
    if (communicator) {
      communicator.terminate();
    }
  }
}
async function performEncoding(communicator, source, config, options) {
  return new Promise((resolve, reject) => {
    let processedFrames = 0;
    let totalFrames;
    const startTime = Date.now();
    const updateProgress = (stage) => {
      if (options?.onProgress) {
        const elapsed = Date.now() - startTime;
        const fps = processedFrames > 0 ? processedFrames / elapsed * 1e3 : 0;
        const percent = totalFrames ? processedFrames / totalFrames * 100 : 0;
        const estimatedRemainingMs = totalFrames && fps > 0 ? (totalFrames - processedFrames) / fps * 1e3 : void 0;
        const progressInfo = {
          percent,
          processedFrames,
          totalFrames,
          fps,
          stage,
          estimatedRemainingMs
        };
        options.onProgress(progressInfo);
      }
    };
    communicator.on("initialized", () => {
      updateProgress("encoding");
      processVideoSource(communicator, source, config).then(() => {
        updateProgress("finalizing");
        communicator.send("finalize");
      }).catch(reject);
    });
    communicator.on("progress", (data) => {
      processedFrames = data.processedFrames;
      if (data.totalFrames !== void 0) {
        totalFrames = data.totalFrames;
      }
      updateProgress("encoding");
    });
    communicator.on("finalized", (data) => {
      if (data.output) {
        updateProgress("finalizing");
        resolve(data.output);
      } else {
        reject(new EncodeError("encoding-failed", "No output produced"));
      }
    });
    communicator.on("error", (data) => {
      const error = new EncodeError(
        data.errorDetail.type || "encoding-failed",
        data.errorDetail.message || "Worker error",
        data.errorDetail
      );
      reject(error);
    });
    communicator.send("initialize", { config });
  });
}
async function processVideoSource(communicator, source, config) {
  if (Array.isArray(source)) {
    await processFrameArray(communicator, source);
  } else if (source instanceof MediaStream) {
    await processMediaStream(communicator, source, config);
  } else if (Symbol.asyncIterator in source) {
    await processAsyncIterable(communicator, source);
  } else {
    throw new EncodeError("invalid-input", "VideoFile processing not yet implemented");
  }
}
async function processFrameArray(communicator, frames) {
  for (let i = 0; i < frames.length; i++) {
    const frame = frames[i];
    const timestamp = i * 1e6 / 30;
    await addFrameToWorker(communicator, frame, timestamp);
  }
}
async function processAsyncIterable(communicator, source) {
  let frameIndex = 0;
  for await (const frame of source) {
    const timestamp = frameIndex * 1e6 / 30;
    await addFrameToWorker(communicator, frame, timestamp);
    frameIndex++;
  }
}
async function processMediaStream(communicator, stream, _config) {
  const videoTracks = stream.getVideoTracks();
  const audioTracks = stream.getAudioTracks();
  const readers = [];
  const processingPromises = [];
  try {
    if (videoTracks.length > 0) {
      const videoTrack = videoTracks[0];
      const processor = new MediaStreamTrackProcessor({ track: videoTrack });
      const reader = processor.readable.getReader();
      readers.push(reader);
      processingPromises.push(
        processVideoReader(communicator, reader)
      );
    }
    if (audioTracks.length > 0) {
      const audioTrack = audioTracks[0];
      const processor = new MediaStreamTrackProcessor({ track: audioTrack });
      const reader = processor.readable.getReader();
      readers.push(reader);
      processingPromises.push(
        processAudioReader(communicator, reader)
      );
    }
    await Promise.all(processingPromises);
  } finally {
    for (const reader of readers) {
      try {
        reader.cancel();
      } catch (e) {
      }
    }
    for (const track of [...videoTracks, ...audioTracks]) {
      track.stop();
    }
  }
}
async function processVideoReader(communicator, reader) {
  try {
    while (true) {
      const { value, done } = await reader.read();
      if (done || !value) break;
      try {
        await addFrameToWorker(communicator, value, value.timestamp || 0);
      } finally {
        value.close();
      }
    }
  } catch (error) {
    throw new EncodeError(
      "video-encoding-error",
      `Video stream processing error: ${error instanceof Error ? error.message : String(error)}`,
      error
    );
  }
}
async function processAudioReader(communicator, reader) {
  try {
    while (true) {
      const { value, done } = await reader.read();
      if (done || !value) break;
      try {
        communicator.send("addAudioData", {
          audio: value,
          timestamp: value.timestamp || 0,
          format: "f32",
          sampleRate: value.sampleRate,
          numberOfFrames: value.numberOfFrames,
          numberOfChannels: value.numberOfChannels
        });
      } finally {
        value.close();
      }
    }
  } catch (error) {
    throw new EncodeError(
      "audio-encoding-error",
      `Audio stream processing error: ${error instanceof Error ? error.message : String(error)}`,
      error
    );
  }
}
async function addFrameToWorker(communicator, frame, timestamp) {
  const videoFrame = await convertToVideoFrame(frame, timestamp);
  try {
    communicator.send("addVideoFrame", {
      frame: videoFrame,
      timestamp
    });
  } finally {
    if (videoFrame !== frame) {
      videoFrame.close();
    }
  }
}
async function convertToVideoFrame(frame, timestamp) {
  if (frame instanceof VideoFrame) {
    return frame;
  }
  if (frame instanceof HTMLCanvasElement) {
    return new VideoFrame(frame, { timestamp });
  }
  if (frame instanceof OffscreenCanvas) {
    return new VideoFrame(frame, { timestamp });
  }
  if (frame instanceof ImageBitmap) {
    return new VideoFrame(frame, { timestamp });
  }
  if (frame instanceof ImageData) {
    return new VideoFrame(frame.data, {
      format: "RGBA",
      codedWidth: frame.width,
      codedHeight: frame.height,
      timestamp
    });
  }
  if (frame && typeof frame === "object") {
    if ("width" in frame && "height" in frame && "data" in frame) {
      const imageDataLike = frame;
      return new VideoFrame(imageDataLike.data, {
        format: "RGBA",
        codedWidth: imageDataLike.width,
        codedHeight: imageDataLike.height,
        timestamp
      });
    }
    if ("width" in frame && "height" in frame && ("getContext" in frame || "transferToImageBitmap" in frame)) {
      return new VideoFrame(frame, { timestamp });
    }
    if ("width" in frame && "height" in frame && "close" in frame && typeof frame.close === "function") {
      return new VideoFrame(frame, { timestamp });
    }
  }
  throw new EncodeError("invalid-input", `Unsupported frame type: ${typeof frame}. Frame must be VideoFrame, HTMLCanvasElement, OffscreenCanvas, ImageBitmap, or ImageData.`);
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  encode
});
//# sourceMappingURL=encode.cjs.map