interface EncoderConfig {
    width: number;
    height: number;
    frameRate: number;
    videoBitrate: number;
    audioBitrate: number;
    /**
     * Controls bitrate distribution for AAC. "constant" produces constant
     * bitrate (CBR) output while "variable" enables variable bitrate (VBR).
     * Not all browsers respect this setting. Chrome 119+ improves CBR support.
     */
    audioBitrateMode?: "constant" | "variable";
    sampleRate: number;
    channels: number;
    container?: "mp4" | "webm";
    codec?: {
        video?: "avc" | "hevc" | "vp9" | "vp8" | "av1";
        audio?: "aac" | "opus";
    };
    /**
     * Optional codec string overrides passed directly to the encoders.
     * For example: `{ video: 'avc1.640028', audio: 'mp4a.40.2' }`.
     */
    codecString?: {
        video?: string;
        audio?: string;
    };
    latencyMode?: "quality" | "realtime";
    /** Preference for hardware or software encoding. */
    hardwareAcceleration?: "prefer-hardware" | "prefer-software" | "no-preference";
    /** Drop new video frames when the number of queued frames exceeds `maxQueueDepth`. */
    dropFrames?: boolean;
    /** Maximum number of queued video frames before dropping. Defaults to `Infinity`. */
    maxQueueDepth?: number;
    /** Total frames for progress calculation if known in advance. */
    totalFrames?: number;
    /** Force a key frame every N video frames. */
    keyFrameInterval?: number;
    /**
     * How to handle the first timestamp of a track.
     * 'offset': Offsets all timestamps so the first one is 0.
     * 'strict': Requires the first timestamp to be 0 (default).
     */
    firstTimestampBehavior?: "offset" | "strict";
    /** Additional VideoEncoder configuration overrides. */
    videoEncoderConfig?: Partial<VideoEncoderConfig>;
    /** Additional AudioEncoder configuration overrides. */
    audioEncoderConfig?: Partial<AudioEncoderConfig>;
}
/**
 * 新しい関数ファーストAPI用の型定義
 */
type Frame = VideoFrame | HTMLCanvasElement | OffscreenCanvas | ImageBitmap | ImageData;
interface VideoFile {
    file: File | Blob;
    type: string;
}
type VideoSource = Frame[] | AsyncIterable<Frame> | MediaStream | VideoFile;
type QualityPreset = 'low' | 'medium' | 'high' | 'lossless';
interface VideoConfig {
    codec?: 'avc' | 'hevc' | 'vp9' | 'vp8' | 'av1';
    bitrate?: number;
    hardwareAcceleration?: 'no-preference' | 'prefer-hardware' | 'prefer-software';
    latencyMode?: 'quality' | 'realtime';
    keyFrameInterval?: number;
}
interface AudioConfig {
    codec?: 'aac' | 'opus';
    bitrate?: number;
    sampleRate?: number;
    channels?: number;
    bitrateMode?: 'constant' | 'variable';
}
interface ProgressInfo {
    percent: number;
    processedFrames: number;
    totalFrames?: number;
    fps: number;
    stage: string;
    estimatedRemainingMs?: number;
}
interface EncodeOptions {
    width?: number;
    height?: number;
    frameRate?: number;
    quality?: QualityPreset;
    video?: VideoConfig;
    audio?: AudioConfig | false;
    container?: 'mp4' | 'webm';
    onProgress?: (progress: ProgressInfo) => void;
    onError?: (error: EncodeError) => void;
}
type EncodeErrorType = 'not-supported' | 'initialization-failed' | 'configuration-error' | 'invalid-input' | 'encoding-failed' | 'video-encoding-error' | 'audio-encoding-error' | 'muxing-failed' | 'cancelled' | 'timeout' | 'worker-error' | 'filesystem-error' | 'unknown';
declare class EncodeError extends Error {
    type: EncodeErrorType;
    cause?: unknown;
    constructor(type: EncodeErrorType, message: string, cause?: unknown);
}

export type { EncodeOptions as E, VideoSource as V, EncoderConfig as a };
