// src/types.ts
var EncodeError = class _EncodeError extends Error {
  constructor(type, message, cause) {
    super(message);
    this.name = "EncodeError";
    this.type = type;
    this.cause = cause;
    Object.setPrototypeOf(this, _EncodeError.prototype);
  }
};

// src/utils/worker-manager.ts
var workerInstance = null;
var workerBlobUrl = null;
function getWorker() {
  if (!workerInstance) {
    workerInstance = createInlineWorker();
  }
  return workerInstance;
}
function terminateWorker() {
  if (workerInstance) {
    workerInstance.terminate();
    workerInstance = null;
  }
  if (workerBlobUrl) {
    URL.revokeObjectURL(workerBlobUrl);
    workerBlobUrl = null;
  }
}
function createInlineWorker() {
  try {
    const workerSource = getWorkerSource();
    const blob = new Blob([workerSource], { type: "application/javascript" });
    workerBlobUrl = URL.createObjectURL(blob);
    const worker = new Worker(workerBlobUrl, { type: "module" });
    worker.onerror = (event) => {
      console.error("Worker error:", event);
      throw new EncodeError("worker-error", `Worker error: ${event.message}`);
    };
    return worker;
  } catch (error) {
    throw new EncodeError(
      "initialization-failed",
      "Failed to create inline worker",
      error
    );
  }
}
function getWorkerSource() {
  return `
    // WebCodecs Encoder Worker (Inline)
    
    // Import statements for WebCodecs APIs
    // Note: In a real implementation, this would be the actual worker.ts content
    
    const mp4Muxer = null; // Placeholder - would import mp4-muxer
    const webmMuxer = null; // Placeholder - would import webm-muxer
    
    let videoEncoder = null;
    let audioEncoder = null;
    let muxer = null;
    let config = null;
    
    self.onmessage = async function(event) {
      const { type, ...data } = event.data;
      
      try {
        switch (type) {
          case 'initialize':
            config = data.config;
            await initialize(config);
            self.postMessage({ type: 'initialized' });
            break;
            
          case 'addVideoFrame':
            if (videoEncoder) {
              await encodeVideoFrame(data.frame, data.timestamp);
            }
            break;
            
          case 'addAudioData':
            if (audioEncoder) {
              await encodeAudioData(data);
            }
            break;
            
          case 'finalize':
            const result = await finalize();
            self.postMessage({ type: 'finalized', output: result });
            break;
            
          case 'cancel':
            cleanup();
            self.postMessage({ type: 'cancelled' });
            break;
            
          default:
            console.warn('Unknown message type:', type);
        }
      } catch (error) {
        self.postMessage({ 
          type: 'error', 
          errorDetail: {
            message: error.message,
            type: 'encoding-failed',
            stack: error.stack
          }
        });
      }
    };
    
    async function initialize(config) {
      // Initialize encoders and muxer based on config
      // This is a placeholder - actual implementation would be much more complex
      console.log('Worker initialized with config:', config);
    }
    
    async function encodeVideoFrame(frame, timestamp) {
      // Encode video frame
      console.log('Encoding video frame at timestamp:', timestamp);
    }
    
    async function encodeAudioData(data) {
      // Encode audio data
      console.log('Encoding audio data:', data);
    }
    
    async function finalize() {
      // Finalize encoding and return result
      console.log('Finalizing encoding');
      return new Uint8Array(0); // Placeholder
    }
    
    function cleanup() {
      // Clean up encoders and muxer
      if (videoEncoder) {
        videoEncoder.close();
        videoEncoder = null;
      }
      if (audioEncoder) {
        audioEncoder.close();
        audioEncoder = null;
      }
      muxer = null;
    }
  `;
}
var WorkerCommunicator = class {
  constructor() {
    this.messageHandlers = /* @__PURE__ */ new Map();
    this.worker = getWorker();
    this.worker.onmessage = this.handleMessage.bind(this);
  }
  handleMessage(event) {
    const { type, ...data } = event.data;
    const handler = this.messageHandlers.get(type);
    if (handler) {
      handler(data);
    }
  }
  /**
   * メッセージハンドラーを登録
   */
  on(type, handler) {
    this.messageHandlers.set(type, handler);
  }
  /**
   * メッセージハンドラーを解除
   */
  off(type) {
    this.messageHandlers.delete(type);
  }
  /**
   * ワーカーにメッセージを送信
   */
  send(type, data = {}) {
    this.worker.postMessage({ type, ...data });
  }
  /**
   * 通信を終了
   */
  terminate() {
    this.messageHandlers.clear();
  }
};
export {
  WorkerCommunicator,
  getWorker,
  terminateWorker
};
//# sourceMappingURL=worker-manager.js.map